#
# Copyright 2010 Andrey Churin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from closurebuilder.script import Command, get_commands
from closurebuilder.script import parser as base_parser

class HelpCommand(Command):
    summary = "Display help"
    usage = '[COMMAND]'

    max_args = 1

    parser = Command.standard_parser()

    def command(self):
        if not self.args:
            self.generic_help()
            return

        name = self.args[0]
        commands = get_commands()
        if name not in commands:
            print 'No such command: %s' % name
            self.generic_help()
            return

        command = commands[name].load()
        runner = command(name)
        runner.run(['-h'])

    def generic_help(self):
        base_parser.print_help()
        print
        commands_grouped = {}
        commands = get_commands()
        longest = max([len(n) for n in commands.keys()])
        for name, command in commands.items():
            try:
                command = command.load()
            except Exception, e:
                print 'Cannot load command %s: %s' % (name, e)
                continue
            if getattr(command, 'hidden', False):
                continue
            commands_grouped.setdefault(
                command.group_name, []).append((name, command))
        commands_grouped = commands_grouped.items()
        commands_grouped.sort()
        print 'Commands:'
        for group, commands in commands_grouped:
            if group:
                print group + ':'
            commands.sort()
            for name, command in commands:
                print '  %s  %s' % (self.pad(name, length=longest),
                                    command.summary)
            print
