#
# Copyright 2010 Andrey Churin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"Paster closure:config command"

import os
import re
import string
import pkg_resources
import email.parser
import datetime

from paste.script import pluginlib
from paste.script.filemaker import FileOp
from tempita import paste_script_template_renderer
from closurebuilder.util import Error
from closurebuilder.manifest import read_manifest

__all__ = ['Metadata']


class Metadata(object):
    _metadata = None

    file_op = FileOp(source_dir=('closurebuilder', 'templates'))

    @classmethod
    def get_closure_meta(cls):
        egg_info = pluginlib.find_egg_info_dir(os.getcwd())

        try:
            parser = email.parser.Parser()
            meta = parser.parse(open(os.path.join(egg_info, 'PKG-INFO')))
        except OSError:
            raise Error('Egg-info is corrupted (%s) \n'
                             '  "PKG-INFO" file is missing' % egg_info)
        return meta

    @classmethod
    def get_closure_dir(cls):
        egg_info = pluginlib.find_egg_info_dir(os.getcwd())

        try:
            f = open(os.path.join(egg_info, 'top_level.txt'))
        except OSError:
            raise Error('Egg-info is corrupted (%s) \n'
                             '  "top_level.txt" file is missing' % egg_info)

        packages = [l.strip() for l in f.readlines()
                    if l.strip() and not l.strip().startswith('#')]
        if not len(packages):
            raise Error('No top level dir found for "closure"')
        f.close()

        base = os.path.dirname(egg_info)
        possible = []
        for pkg in packages:
            d = os.path.join(base, pkg, 'closure')
            if os.path.exists(d):
                possible.append((pkg, d))

        if len(possible) > 1:
            raise Error('Multiple "closure" dirs found (%s)' % possible)

        pkg = packages[0]
        path = os.path.join(base, pkg, 'closure')
        if not possible:
            cls.file_op.ensure_dir(path, package=False)
        return path

    @classmethod
    def get_instance(cls):
        if cls._metadata:
            return cls._metadata

        metadata = Metadata()
        metadata.closure_dir = cls.get_closure_dir()

        cls._metadata = metadata
        return metadata

    def configure(self):
        from closurebuilder import sources, jscompiler

        compiler_settings = os.path.join(self.closure_dir,
                                            'compiler-settings.txt')
        sources_txt = os.path.join(self.closure_dir, 'sources.txt')

        self.compiler = read_manifest(filename = compiler_settings,
                            processor = jscompiler.Compiler(self.closure_dir))
        self.sources = read_manifest(filename = sources_txt,
                                processor = sources.Sources(self.closure_dir))
        self.sources.include_strictly(pkg_resources.resource_filename(
                                            'closurebuilder', 'closurelibs'))
        meta = self.get_closure_meta()
        self.template_vars = self.file_op.template_vars
        self.template_vars.update({
            'author': meta.get('Author'),
            'author_email': meta.get('Author-email'),
            'year': datetime.date.today().year,
            'license': meta.get('License')
        })

        try:
            license = self.file_op.load_content('', '', '',
                        os.path.join('licenses',
                            meta.get('License').replace(' ', '_') + '_tmpl'),
                            template_renderer=paste_script_template_renderer)
        except:
            print 'Template file for license "%s" not found'% \
                        meta.get('License')
            license = self.file_op.load_content('', '', '',
                        os.path.join('licenses', 'unknown_license_tmpl'),
                            template_renderer=paste_script_template_renderer)

        self.template_vars.update({
            'license': license
        })

    @classmethod
    def copy_file(cls, template, dest, filename, override = True):
        if override == False and os.path.exists(os.path.join(dest, filename)):
            return

        content = cls.file_op.load_content(None, '*', '*', template,
                            template_renderer=paste_script_template_renderer)
        cls.file_op.ensure_file(os.path.join(dest, filename), content, False)
