#
# Copyright 2010 Andrey Churin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
import sys
import email.parser
import optparse

from closurebuilder.metadata import Metadata
from closurebuilder.util import Error
from paste.script import pluginlib
from paste.script.command import BadCommand, Command

__all__ = ['run']

parser = optparse.OptionParser(add_help_option=False,
                               usage='%prog COMMAND [command_options]')

parser.add_option(
    '-h', '--help',
    action='store_true',
    dest='do_help',
    help="Show this help message")

parser.disable_interspersed_args()

system_plugins = []

@classmethod
def get_closure_meta(cls):
    path = cls.get_closure_dir()
    try:
        parser = email.parser.Parser()
        meta = parser.parse(open(os.path.join(path, 'closure-top.txt')))
    except OSError:
        raise Error('Cannot open file %s' %
                        os.path.join(path, 'closure-top.txt'))
    return meta

@classmethod
def get_closure_dir(cls):
    dir = os.getcwd()
    while 1:
        try:
            filenames = os.listdir(dir)
        except OSError:
            return os.getcwd()
        for fn in filenames:
            path = os.path.join(dir, fn, 'closure-top.txt')
            if os.path.isfile(path):
                return os.path.join(dir, fn)
        parent = os.path.dirname(dir)
        if parent == dir:
            return os.getcwd()
        dir = parent

def run(args=None):
    if args is None:
        args = sys.argv[1:]
    sys.argv[0] = os.path.split(sys.argv[0])[1]
    Metadata.get_closure_dir = get_closure_dir
    Metadata.get_closure_meta = get_closure_meta
    options, args = parser.parse_args(args)
    commands = get_commands()
    if options.do_help:
        args = ['help'] + args
    if not args:
        args = ['help']
    command_name = args[0]
    if command_name not in commands:
        print 'Invalid command name %s' %command_name
        sys.exit(1)
    else:
        command = commands[command_name].load()
    invoke(command, command_name, options, args[1:])

def get_commands():
    plugins = pluginlib.resolve_plugins(['ClosureBuilder'])
    commands = pluginlib.load_commands_from_plugins(plugins)
    return commands

def invoke(command, command_name, options, args):
    try:
        runner = command(command_name)
        exit_code = runner.run(args)
    except BadCommand, e:
        print e.message
        exit_code = e.exit_code
    sys.exit(exit_code)
